#pragma once
#ifndef PLAYER_MOVEMENT_H
#define PLAYER_MOVEMENT_H

#include <chrono>
#include <thread>
#include <iostream>
#include "sgg/graphics.h"
#define M_PI 3.14159265358979323846
// Player properties and state
struct Player {
    float x;             // Player's x-axis position
    float y;             // Player's y-axis position
    float width, height; ///< Dimensions of the player (width and height).
    float speed;         // Current speed of the player
    float angle;        ///< Orientation of the player in degrees.
    bool isJumping;      // Indicates if the player is currently jumping
    bool isCrouching;    // Indicates if the player is crouching
    const float defaultHeight = 1.0f;  // Default height for standing position
    const float crouchHeight = 0.5f;   // Height when crouching
    
    graphics::Brush frontBrush; ///< Color used for the player's front section.
    graphics::Brush backBrush;
};

/**
 * Delays keyboard response by a specified amount of time.
 * This function pauses the program's execution to simulate a "slowed" input response,
 * effectively ignoring any keyboard inputs during the delay period.
 *
 * @param delay_ms The delay duration in milliseconds before accepting keyboard input.
 *
 * @note This function can be used to limit input frequency in scenarios where
 * rapid input responses are undesired.
 */
void slowKeyboardInput(int delay_ms) {
    // Ensure the delay is positive
    if (delay_ms > 0) {
        // Put the program to sleep for the specified duration
        std::this_thread::sleep_for(std::chrono::milliseconds(delay_ms));
    }
}

/**
 * Makes the player jump along the y-axis.
 * @param player Reference to the Player struct.
 * @param jumpHeight The height of the jump.
 * This function updates the player's position along the y-axis
 * by increasing the y-coordinate to simulate a jump.
 */
void jumpY(Player& player, float jumpHeight) {
    if (!player.isJumping) {
        player.y += jumpHeight;
        player.isJumping = true;
        std::cout << "Player jumped on Y-axis to: " << player.y << std::endl;
    }
    player.isJumping = false;
}

/**
 * Makes the player jump along the x-axis.
 * @param player Reference to the Player struct.
 * @param jumpDistance The distance of the jump.
 * This function updates the player's position along the x-axis
 * by increasing the x-coordinate to simulate a jump.
 */
void jumpX(Player& player, float jumpDistance) {
    if (!player.isJumping) {
        player.x += jumpDistance;
        player.isJumping = true;
        std::cout << "Player jumped on X-axis to: " << player.x << std::endl;
    }
    player.isJumping = false;
}


/**
 * Increases the player's speed for a short duration, simulating a sprint.
 * @param player Reference to the Player struct.
 * @param sprintMultiplier Multiplier applied to the player's speed.
 * The function applies a temporary speed increase to the player,
 * giving them a boost for faster movement.
 */
void sprint(Player& player, float sprintMultiplier) {
    float originalSpeed = player.speed;
    player.speed *= sprintMultiplier;
    std::cout << "Player is sprinting at speed: " << player.speed << std::endl;

    // Reset speed to normal after a short duration (simple simulation)
    //player.speed = originalSpeed;
}

/**
 * Slows down the player's movement speed for a stealthier approach.
 * @param player Reference to the Player struct.
 * @param walkMultiplier Multiplier applied to reduce the player's speed.
 * This function applies a slower speed to the player, making movement more cautious.
 */
void walk(Player& player, float walkMultiplier) {
    float originalSpeed = player.speed;
    player.speed *= walkMultiplier;
    std::cout << "Player is walking at speed: " << player.speed << std::endl;

    // Reset speed to normal after walking
    //player.speed = originalSpeed;
}

// Movement and rotation functions

/**
 * @brief Moves the player forward in the direction they are facing.
 *
 * This function updates the player’s position based on their speed and orientation.
 * @param player The player to be moved forward.
 */
void moveForward(Player& player) {
    player.x += player.speed * cos(player.angle * M_PI / 180.0f);
    player.y += player.speed * sin(player.angle * M_PI / 180.0f);
}

/**
 * @brief Moves the player backward in the opposite direction they are facing.
 *
 * This function updates the player’s position to move them backward relative to their current orientation.
 * @param player The player to be moved backward.
 */
void moveBackward(Player& player) {
    player.x -= player.speed * cos(player.angle * M_PI / 180.0f);
    player.y -= player.speed * sin(player.angle * M_PI / 180.0f);
}

/**
 * @brief Rotates the player to the left by a specified degree.
 *
 * This function adjusts the player’s orientation to the left.
 * @param player The player to be rotated.
 * @param degrees The amount in degrees to rotate the player left.
 */
void rotateLeft(Player& player, float degrees) {
    player.angle -= degrees;
    if (player.angle < 0) player.angle += 360;
}

/**
 * @brief Rotates the player to the right by a specified degree.
 *
 * This function adjusts the player’s orientation to the right.
 * @param player The player to be rotated.
 * @param degrees The amount in degrees to rotate the player right.
 */
void rotateRight(Player& player, float degrees) {
    player.angle += degrees;
    if (player.angle >= 360) player.angle -= 360;
}

// Position and boundary checking functions

/**
 * @brief Checks if the player is within the boundaries of the window.
 *
 * This function verifies that the player is within the visible window area, considering
 * the player's dimensions and position.
 * @param player The player to check.
 * @param windowWidth The width of the window boundary.
 * @param windowHeight The height of the window boundary.
 * @return true if the player is within bounds; otherwise false.
 */
bool isWithinBounds(const Player& player, float windowWidth, float windowHeight) {
    return player.x >= player.width / 2 && player.x <= windowWidth - player.width / 2 &&
        player.y >= player.height / 2 && player.y <= windowHeight - player.height / 2;
}

/**
 * @brief Ensures that the player stays within the window boundaries.
 *
 * This function adjusts the player’s position if they exceed the window’s bounds.
 * @param player The player to enforce bounds on.
 * @param windowWidth The width of the window boundary.
 * @param windowHeight The height of the window boundary.
 */
void enforceBounds(Player& player, float windowWidth, float windowHeight) {
    if (player.x < player.width / 2) player.x = player.width / 2;
    if (player.x > windowWidth - player.width / 2) player.x = windowWidth - player.width / 2;
    if (player.y < player.height / 2) player.y = player.height / 2;
    if (player.y > windowHeight - player.height / 2) player.y = windowHeight - player.height / 2;
}

// Graphical representation functions

/**
 * @brief Renders the player as a rectangle with a distinguishable front and back.
 *
 * The player's orientation is taken into account, with a slight offset to visually indicate the front of the player.
 * @param player The player to render.
 */
void drawPlayer(const Player& player) {
    graphics::setOrientation(player.angle); // Set orientation to match the player's angle

    // Define back brush color to distinguish the back
    graphics::Brush backBrush = player.backBrush;
    backBrush.fill_color[0] = 0.6f; // A darker shade for the back
    backBrush.fill_color[1] = 0.6f;
    backBrush.fill_color[2] = 0.6f;

    // Define front brush color for distinction
    graphics::Brush frontBrush = player.frontBrush;
    frontBrush.fill_color[0] = 1.0f; // A lighter shade for the front
    frontBrush.fill_color[1] = 0.8f;
    frontBrush.fill_color[2] = 0.0f;

    // Draw the back side of the player rectangle
    graphics::drawRect(player.x, player.y, player.width, player.height, backBrush);

    // Draw the front side with a slight offset for visual distinction
    float frontOffset = player.width * 0.2f;
    float frontX = player.x + frontOffset * cos(player.angle * M_PI / 180.0f);
    float frontY = player.y + frontOffset * sin(player.angle * M_PI / 180.0f);
    graphics::drawRect(frontX, frontY, player.width * 0.8f, player.height * 0.8f, frontBrush); // Slightly smaller for emphasis

    graphics::resetPose(); // Reset orientation after drawing
}

#endif // PLAYER_MOVEMENT_H
