#pragma once
#ifndef PLAYER_MOVEMENT_ES_H
#define PLAYER_MOVEMENT_ES_H

#include <chrono>
#include <thread>
#include <iostream>
#include "sgg/graphics.h"
#define M_PI 3.14159265358979323846

// Propiedades y estado del jugador
struct Player {
    float x;             // Posición del jugador en el eje x
    float y;             // Posición del jugador en el eje y
    float width, height; ///< Dimensiones del jugador (ancho y alto).
    float speed;         // Velocidad actual del jugador
    float angle;         ///< Orientación del jugador en grados.
    bool isJumping;      // Indica si el jugador está saltando actualmente
    bool isCrouching;    // Indica si el jugador está agachado
    const float defaultHeight = 1.0f;  // Altura predeterminada para la posición de pie
    const float crouchHeight = 0.5f;   // Altura cuando está agachado

    graphics::Brush frontBrush; ///< Color utilizado para la parte frontal del jugador.
    graphics::Brush backBrush;
};

/**
 * Retarda la respuesta del teclado por un tiempo especificado.
 * Esta función pausa la ejecución del programa para simular una respuesta "lenta" a las entradas del teclado,
 * ignorando efectivamente cualquier entrada de teclado durante el período de demora.
 *
 * @param delay_ms Duración del retardo en milisegundos antes de aceptar la entrada del teclado.
 *
 * @nota Esta función se puede usar para limitar la frecuencia de entrada en escenarios donde
 * se desean respuestas de entrada rápidas.
 */
void slowKeyboardInput(int delay_ms) {
    // Asegura que el retardo sea positivo
    if (delay_ms > 0) {
        // Pone el programa en espera durante la duración especificada
        std::this_thread::sleep_for(std::chrono::milliseconds(delay_ms));
    }
}

/**
 * Hace que el jugador salte a lo largo del eje y.
 * @param player Referencia a la estructura Player.
 * @param jumpHeight La altura del salto.
 * Esta función actualiza la posición del jugador a lo largo del eje y
 * aumentando la coordenada y para simular un salto.
 */
void jumpY(Player& player, float jumpHeight) {
    if (!player.isJumping) {
        player.y += jumpHeight;
        player.isJumping = true;
        std::cout << "El jugador saltó en el eje Y a: " << player.y << std::endl;
    }
    player.isJumping = false;
}

/**
 * Hace que el jugador salte a lo largo del eje x.
 * @param player Referencia a la estructura Player.
 * @param jumpDistance La distancia del salto.
 * Esta función actualiza la posición del jugador a lo largo del eje x
 * aumentando la coordenada x para simular un salto.
 */
void jumpX(Player& player, float jumpDistance) {
    if (!player.isJumping) {
        player.x += jumpDistance;
        player.isJumping = true;
        std::cout << "El jugador saltó en el eje X a: " << player.x << std::endl;
    }
    player.isJumping = false;
}

/**
 * Aumenta la velocidad del jugador por un corto tiempo, simulando un sprint.
 * @param player Referencia a la estructura Player.
 * @param sprintMultiplier Multiplicador aplicado a la velocidad del jugador.
 * La función aplica un aumento temporal de velocidad al jugador,
 * dándole un impulso para moverse más rápido.
 */
void sprint(Player& player, float sprintMultiplier) {
    float originalSpeed = player.speed;
    player.speed *= sprintMultiplier;
    std::cout << "El jugador está corriendo a velocidad: " << player.speed << std::endl;
}

/**
 * Ralentiza la velocidad de movimiento del jugador para un enfoque más sigiloso.
 * @param player Referencia a la estructura Player.
 * @param walkMultiplier Multiplicador aplicado para reducir la velocidad del jugador.
 * Esta función aplica una velocidad más lenta al jugador, haciendo que el movimiento sea más cauteloso.
 */
void walk(Player& player, float walkMultiplier) {
    float originalSpeed = player.speed;
    player.speed *= walkMultiplier;
    std::cout << "El jugador está caminando a velocidad: " << player.speed << std::endl;
}

// Funciones de movimiento y rotación

/**
 * @brief Mueve al jugador hacia adelante en la dirección en la que está mirando.
 *
 * Esta función actualiza la posición del jugador en función de su velocidad y orientación.
 * @param player El jugador que se moverá hacia adelante.
 */
void moveForward(Player& player) {
    player.x += player.speed * cos(player.angle * M_PI / 180.0f);
    player.y += player.speed * sin(player.angle * M_PI / 180.0f);
}

/**
 * @brief Mueve al jugador hacia atrás en la dirección opuesta a la que está mirando.
 *
 * Esta función actualiza la posición del jugador para moverlo hacia atrás en relación con su orientación actual.
 * @param player El jugador que se moverá hacia atrás.
 */
void moveBackward(Player& player) {
    player.x -= player.speed * cos(player.angle * M_PI / 180.0f);
    player.y -= player.speed * sin(player.angle * M_PI / 180.0f);
}

/**
 * @brief Rota al jugador hacia la izquierda en una cantidad de grados especificada.
 *
 * Esta función ajusta la orientación del jugador hacia la izquierda.
 * @param player El jugador que se rotará.
 * @param degrees La cantidad en grados para rotar al jugador hacia la izquierda.
 */
void rotateLeft(Player& player, float degrees) {
    player.angle -= degrees;
    if (player.angle < 0) player.angle += 360;
}

/**
 * @brief Rota al jugador hacia la derecha en una cantidad de grados especificada.
 *
 * Esta función ajusta la orientación del jugador hacia la derecha.
 * @param player El jugador que se rotará.
 * @param degrees La cantidad en grados para rotar al jugador hacia la derecha.
 */
void rotateRight(Player& player, float degrees) {
    player.angle += degrees;
    if (player.angle >= 360) player.angle -= 360;
}

// Funciones de verificación de posición y límites

/**
 * @brief Verifica si el jugador está dentro de los límites de la ventana.
 *
 * Esta función verifica que el jugador esté dentro del área visible de la ventana, considerando
 * las dimensiones y posición del jugador.
 * @param player El jugador a verificar.
 * @param windowWidth El ancho del límite de la ventana.
 * @param windowHeight La altura del límite de la ventana.
 * @return true si el jugador está dentro de los límites; de lo contrario, false.
 */
bool isWithinBounds(const Player& player, float windowWidth, float windowHeight) {
    return player.x >= player.width / 2 && player.x <= windowWidth - player.width / 2 &&
        player.y >= player.height / 2 && player.y <= windowHeight - player.height / 2;
}

/**
 * @brief Asegura que el jugador se mantenga dentro de los límites de la ventana.
 *
 * Esta función ajusta la posición del jugador si excede los límites de la ventana.
 * @param player El jugador al que se le aplicarán los límites.
 * @param windowWidth El ancho del límite de la ventana.
 * @param windowHeight La altura del límite de la ventana.
 */
void enforceBounds(Player& player, float windowWidth, float windowHeight) {
    if (player.x < player.width / 2) player.x = player.width / 2;
    if (player.x > windowWidth - player.width / 2) player.x = windowWidth - player.width / 2;
    if (player.y < player.height / 2) player.y = player.height / 2;
    if (player.y > windowHeight - player.height / 2) player.y = windowHeight - player.height / 2;
}

// Funciones de representación gráfica

/**
 * @brief Renderiza al jugador como un rectángulo con una parte frontal y trasera distinguibles.
 *
 * La orientación del jugador se toma en cuenta, con un ligero desplazamiento para indicar visualmente la parte frontal del jugador.
 * @param player El jugador a renderizar.
 */
void drawPlayer(const Player& player) {
    graphics::setOrientation(player.angle); // Establece la orientación para coincidir con el ángulo del jugador

    // Definir el color del pincel trasero para distinguir la parte trasera
    graphics::Brush backBrush = player.backBrush;
    backBrush.fill_color[0] = 0.6f; // Un tono más oscuro para la parte trasera
    backBrush.fill_color[1] = 0.6f;
    backBrush.fill_color[2] = 0.6f;

    // Definir el color del pincel frontal para distinción
    graphics::Brush frontBrush = player.frontBrush;
    frontBrush.fill_color[0] = 1.0f; // Un tono más claro para la parte frontal
    frontBrush.fill_color[1] = 0.8f;
    frontBrush.fill_color[2] = 0.0f;

    // Dibuja la parte trasera del rectángulo del jugador
    graphics::drawRect(player.x, player.y, player.width, player.height, backBrush);

    // Dibuja la parte frontal con un ligero desplazamiento para distinción visual
    float frontOffset = player.width * 0.2f;
    float frontX = player.x + frontOffset * cos(player.angle * M_PI / 180.0f);
    float frontY = player.y + frontOffset * sin(player.angle * M_PI / 180.0f);
    graphics::drawRect(frontX, frontY, player.width * 0.8f, player.height * 0.8f, frontBrush); // Ligeramente más pequeño para énfasis

    graphics::resetPose(); // Restablece la orientación después de dibujar
}

#endif // PLAYER_MOVEMENT_ES_H
